@@ -13,7 +13,19 @@ from .cmd_submit_build import parse_complete_tag
 from .exceptions import InvalidConfiguration
 from .utils import indent, safe_yaml_dump, wrap_config_reader2
 
-__all__ = ["ChallengeDescription"]
+__all__ = [
+    "ChallengeDescription",
+    "EvaluationParameters",
+    "ChallengesConstants",
+    "ChallengeStep",
+    "ChallengeTransitions",
+    "ChallengeDependency",
+    "InvalidChallengeDescription",
+    "Scoring",
+    "Score",
+    "SubmissionDescription",
+    "ServiceDefinition",
+]
 
 
 class InvalidChallengeDescription(Exception):
@@ -142,7 +154,7 @@ class ServiceDefinition:
                 s = yaml.safe_dump(v)
                 environment[k] = s
             else:
-                msg = 'The type %s is not allowed for environment variable "%s".' % (type(v).__name__, k,)
+                msg = f'The type {type(v).__name__} is not allowed for environment variable "{k}".'
                 raise InvalidConfiguration(msg)
         ports_ = d0.pop("ports", [])
         ports = []
@@ -150,7 +162,10 @@ class ServiceDefinition:
             if isinstance(s, int):
                 internal = s
                 external = None
-            else:
+            elif isinstance(s, dict):
+                internal = s.get("internal", None)
+                external = s.get("internal", None)
+            elif isinstance(s, str):
                 if not ":" in s:
                     # raise InvalidConfiguration(s)
                     internal = int(s)
@@ -159,6 +174,8 @@ class ServiceDefinition:
                     tokens = s.split(":")
                     external = int(tokens[0])
                     internal = int(tokens[1])
+            else:
+                raise ValueError(repr(ports_))
             ports.append(PortDefinition(external=external, internal=internal))
         return ServiceDefinition(image=image, environment=environment, build=build, ports=ports)
 
@@ -353,17 +370,17 @@ class ChallengeTransitions:
         ts = []
         for t in self.transitions:
             if t.first == STATE_START:
-                ts.append("At the beginning execute step `%s`." % t.second)
+                ts.append(f"At the beginning execute step `{t.second}`.")
             else:
                 if t.second in [STATE_ERROR, STATE_FAILED, STATE_SUCCESS]:
                     ts.append(
-                        "If step `%s` finishes with status `%s`, then declare the submission `%s`."
-                        % (t.first, t.condition, t.second)
+                        f"If step `{t.first}` finishes with status `{t.condition}`, then declare the "
+                        f"submission `{t.second}`."
                     )
                 else:
                     ts.append(
-                        "If step `%s` finishes with status `%s`, then execute step `%s`."
-                        % (t.first, t.condition, t.second)
+                        f"If step `{t.first}` finishes with status `{t.condition}`, then execute step `"
+                        f"{t.second}`."
                     )
         return ts
 
@@ -680,10 +697,6 @@ class ChallengeDescription:
         if self.date_close.tzinfo is None:
             raise ValueError(self.date_close)
 
-    #
-    # def get_steps(self):
-    #     return self.steps
-
     def get_next_steps(self, status):
         return self.ct.get_next_steps(status)
 
